%******************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @since: 03-12-2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Two component Flash tank model
% @input: t = time, w = system variables data = parameters
% @output: F = residuals, G = Generalized Derivative Matrix Q = Heat
% transfer
%******************************************************************

function [F,G,Q] = Flash(t,w,data)

%% Unpacking parameters
par = data.par;
F_in = par.F_in;            % Inlet flow rate [kmol/s]
Q_0 = par.Q;                % Heat transfer [MW]
Q_max = par.Q_max;          % Maximum Heat transfer [MW]
R = par.R;                  % Gas constant [MJ/kmol*kK]
V_T = par.V_T;              % Total volume [m^3]
P_0 = par.P_0;              % Outlet pressure [MPa]
Cp_V_1 = par.Cp_V(1);       % Vapor Heat Capcity comp 1 [MJ/kmol*kK]
Cp_L_1 =  par.Cp_L(1);      % Liquid Heat Capcity comp 1 [MJ/kmol*kK]
Cp_V_2 = par.Cp_V(2);       % Vapor Heat Capcity comp 2 [MJ/kmol*kK]
Cp_L_2 =  par.Cp_L(2);      % Liquid Heat Capcity comp 2 [MJ/kmol*kK]
h_vap_1 = par.h_vap(1);     % Heat of vaporization comp 1 [MJ/kmol]
h_vap_2 = par.h_vap(2);     % Heat of vaporization comp 2 [MJ/kmol]
A_1 = par.Antoine(1,1);     % Antoine Parameter comp 1
B_1 = par.Antoine(2,1);     % Antoine Parameter comp 1
C_1 = par.Antoine(3,1);     % Antoine Parameter comp 1
A_2 = par.Antoine(1,2);     % Antoine Parameter comp 2
B_2 = par.Antoine(2,2);     % Antoine Parameter comp 2
C_2 = par.Antoine(3,2);     % Antoine Parameter comp 2
rho_1 = par.rho(1);         % Molar density comp 1 [Mmol/m^3]
rho_2 = par.rho(1);         % Molar density comp 2 [Mmol/m^3]
z_1 = par.z(1);             % Inlet composition comp 1
z_2 = par.z(2);             % Inlet composition comp 1
h_in = par.h_in;            % Inlet molar enthalpy [MJ/kmol]
C0 = par.C0;                % Compressability factor [1/MPa]
T_ref = par.T_ref;          % Reference temperature [kK]
c_V = par.c_V;              % Vapor valve coefficient [kmol/bar^0.5*s]
c_L = par.c_L;              % Liquid valve coefficient [kmol/bar^0.5*s]

% Defining the changes in heat transfer 
if t < 5
    Q = Q_0;
elseif t < 100
    Q = Q_0 + (Q_max-Q_0)/(100-5)*(t-5);
else
    Q = Q_max;
end
%% Unpacking system variables
w = valder(w,eye(length(w)));
M_1 = w(1);        % Component 1 Molar hold-up [kmol]
M_2 = w(2);        % Component 2 Molar hold-up [kmol]
U = w(3);          % Internal Energy [MJ]
F_V = w(4);        % Vapor flow rate [kmol/s]
F_L = w(5);        % Liquid flow rate [kmol/s]
M_V = w(6);        % Vapor molar hold-up [kmol]
P = w(7);          % Pressure [MPa]
T = w(8);          % Temperature [kK]
V_L = w(9);        % Liquid Volume [m^3]
x_1 = w(10);       % Liquid mole fraction component 1
x_2 = w(11);       % Liquid mole fraction component 2
y_1 = w(12);       % Vapor mole fraction component 1
y_2 = w(13);       % Vapor mole fraction component 2
H = w(14);         % Enthalpy [MJ]
h_L = w(15);       % Molar enthalpy in liquid phase [MJ/kmol]
h_V = w(16);       % Molar enthalpy in vapor phase [MJ/kmol]
V_V = w(17);       % Vapor volume [m^3]
rho_L = w(18);     % Liquid molar density [Mmol/kK]
M_L = w(19);       % Liquid molar hold-up [kmol]
k_1 = w(20);       % Vapor-liquid equilibrium coefficent component 1
k_2 = w(21);       % Vapor-liquid equilibrium coefficent component 2
Ps_1 = w(22);      % Saturation pressure component 1 
Ps_2 = w(23);      % Saturation pressure component 2


%% Differential equations
f1 = F_in*z_1 - F_L*x_1 - F_V * y_1;
f2 = F_in*z_2 - F_L*x_2 - F_V * y_2;
f3 = F_in*h_in - F_L*h_L - F_V*h_V + Q;

%% Algebraic equations
f4 = M_1 - (M_L*x_1 + M_V*y_1);
f5 = M_2 - (M_L*x_2 + M_V*y_2);
f6 = (M_1+M_2) - (M_L + M_V);
f7 = H - (M_L*h_L + M_V*h_V);
f8 = H - (U + P*V_T);
f9 = y_1 - k_1*x_1;
f10 = y_2 - k_2*x_2;
f11 = midobj(M_V/(M_V+M_L),(x_1+x_2) - (y_1+y_2), (M_V/(M_V+M_L))-1);
f12 = V_T - (V_L + V_V);
f13 = P*V_V - M_V*R*T;
f14 = V_L - M_L/(rho_L*1e3);
f15 = F_V*V_T - c_V*V_V*max2(0,(P-P_0)/sqrt(abs((P-P_0))+1e-10));
f16 = F_L*V_T - c_L*V_L*max2(0,(P-P_0)/sqrt(abs((P-P_0))+1e-10));
f17 = 1/rho_L - (x_1/(rho_1*(1+C0*(P-0.1))) + x_2/(rho_2*(1+C0*(P-0.1))));
f18 = h_L - ((x_1*Cp_L_1)*(T-T_ref) + (x_2*Cp_L_2)*(T-T_ref));
f19 = h_V - (y_1*(h_vap_1 + Cp_V_1*(T-T_ref)) + y_2*(h_vap_2 +...
    Cp_V_2*(T-T_ref)));
f20 = k_1*P - Ps_1;
f21 = k_2*P - Ps_2;
f22 = Ps_1 - 10^(A_1 - B_1/(T*1e3+C_1))/10;
f23 = Ps_2 - 10^(A_2 - B_2/(T*1e3+C_2))/10;

%% Extracting the values and the derivatives from the valder objects
F = [getVal(f1);getVal(f2);getVal(f3);getVal(f4);getVal(f5);getVal(f6);...
    getVal(f7);getVal(f8);getVal(f9);getVal(f10);getVal(f11);...
    getVal(f12);getVal(f13);getVal(f14);getVal(f15);getVal(f16);...
    getVal(f17);getVal(f18);getVal(f19);getVal(f20);getVal(f21);...
    getVal(f22);getVal(f23)];

G = [getDer(f1);getDer(f2);getDer(f3);getDer(f4);getDer(f5);getDer(f6);...
    getDer(f7);getDer(f8);getDer(f9);getDer(f10);getDer(f11);...
    getDer(f12);getDer(f13);getDer(f14);getDer(f15);getDer(f16);...
    getDer(f17);getDer(f18);getDer(f19);getDer(f20);getDer(f21);...
    getDer(f22);getDer(f23)];

end