%**************************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @project: Master Thesis 2018
% @since: June 2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Main script for finding convergence data by finding steady
%               state solutions for different values of heat transfer Q
% @estimated run time: 1.5 hours
%**************************************************************************
% Description of files
% @ Flash(t,w,data): Two component flash tank model
% @ HEX(t,w,data): Two component HEX model
% @ initialGuesses_gases(par): Initial guesses and boundaries for two
%                              component HEX in vapor-only region
% @ parmeters: Parameter values two component HEX with water and
%              methanol
% @ plotConvergence(t,w,Q): Function for plotting result from simulation
%                           of two component HEX
% @ getConvergenceData(w,Q): Returns the residual and lamdba at each
%                           iteration
% @ valder: Matlab class for calculating the function residuals and
%           Generalized derivatives.
%**************************************************************************
%**************************************************************************
clear all
clc
tic
%% Variable list
% Var1: M_1    Molar hold up of component 1 (Methanol)     [kmol]
% Var2: M_2    Molar hold up of component 1 (Water)        [kmol]
% Var3 : U      Internal Energy                            [MJ]
% Var4 : F_V    Vapor outlet flow                          [kmol/s]
% Var5 : F_L    Liquid outlet flow                         [kmol/s]
% Var6 : M_V    Molar vapor hold-up                        [kmol]
% Var7 : P      Pressure                                   [MPa]
% Var8 : T      Temperature                                [K]
% Var9 : V_L    Liquid volume                              [m^3]
% Var10: x_1    Liquid fraction of component 1 (Methanol)  [-]
% Var11: x_2    Liquid fraction of component 2 (Water)     [-]
% Var12: y_1    Vapor fraction of component 1 (Methanol)   [-]
% Var13: y_2    Vapor fraction of component 1 (Water)      [-]
% Var14: H      Enthalpy                                   [MJ]
% Var15: h_L    Liquid molar enthalpy                      [MJ/kmol]
% Var16: h_V    Vapor molar enthalpy                       [MJ/kmol]
% Var17: V_V    Vapor volume                               [m^3]
% Var18: rho_L  Liquid molar density                       [Mmol/m^3]
% Var19: M_L    Liquid molar hold-up                       [kmol]
% Var20: K_1    Equilibrium coeffiecient for component 1   [-]
% Var21: K_2    Equilibrium coeffiecient for component 2   [-]
% Var22: Ps_1   Saturation pressure for component 1        [MPa]
% Var23: Ps_2   Saturation pressure for component 2        [MPa]

% Component list:
% 1: Methanol       CH3OH
% 2: Water          H2O

%% Parameters and initial guesses
data.par = parameters();
[w0,lb,ub] = initialGuesses_gases(data.par);

%% Solver settings
options_lsq = optimoptions(@lsqnonlin,'Display','iter',...
     'MaxIterations',500,'MaxFunEvals',1e10,...
     'stepTolerance',1e-20,'FunctionTolerance',1e-20,...
     'specifyObjectiveGradient', true,...
     'OptimalityTolerance',1e-20);
options_fsolve = optimoptions(@fsolve,'Display','iter',...
     'MaxIterations',3e3,'MaxFunEvals',1e10,...
     'stepTolerance',1e-13,'FunctionTolerance',1e-13,...
     'specifyObjectiveGradient', true,...
     'OptimalityTolerance',1e-13, 'Algorithm','levenberg-marquardt');
%% Solving a two component flash tank with no heat transfer
data.par.Q = 0;
data.par.V_T = data.par.V_T/data.par.M; % Volume of Flash tank is equal to 
                                        % 1/M of the HEX
lb(17) = data.par.V_T;                  % Lower boundary on volume
ub(17) = data.par.V_T;                  % Upper boundary on volume
w0 = lsqnonlin(@(w) Flash(1,w,data), w0,lb,ub,options_lsq);
%% Full heat exchanger steady state
data.par = parameters();
w_initial = [];
% Repeating the system variables M times
for i = 1:23
    w_initial = [w_initial;repmat(w0(i),data.par.M,1)];
end
%% Increasing heat removal until the desired initial value
Q = 0:-0.01:-0.08;
data.par.Q = 0;
w_initial(:,1) = fsolve(@(w)HEX(0,w,data)...
        ,w_initial,options_fsolve);
%% Residuals at each iterations
residual = {};                  % Residuals at each iteration
lambda = {};                    % Lambda parameter in LMA at each iter.
iterations = [];                % Nr of iterations
wsol = [];                      % Solution for each value of Q
Q = 0:-0.01:-4.2;
data.par.Q = 0;
[wsol(:,1) residual{1}, lambda{1}]...
    = getConvergenceData(w_initial,data);
for i = 2:length(Q)
    data.par.Q = Q(i);
    [wsol(:,i) residual{i}, lambda{i}]...
        = getConvergenceData(wsol(:,i-1),data);
end
%% Nr of iterations
iterations = zeros(1,length(residual));
for i  = 1:length(residual)
    iterations(i) = length(residual{i});
end
%% Check phase region
max = 0;            % Nr of cells in vapor-liquid region
index = [];         % At which index are there a phase change

j= 1;
for i = 1:length(Q)
    NrTwoPhase(i) = sum(abs((wsol(9*3+1:10*3,i) + wsol(10*3+1:11*3,i)) -...
        (wsol(11*3+1:12*3,i) + wsol(12*3+1:13*3,i))) < 1e-8);
    if NrTwoPhase(i) > max
        index(j) = i;
        j = j+1;
        max = NrTwoPhase(i);
    end
    if NrTwoPhase(i) < max
        index(j) = i;
        j = j+1;
        max = NrTwoPhase(i);
    end
end
%% Condition Number
for i = 1:length(wsol)
    [~,G] = HEX(1,wsol(:,i),data);
    condNr(i) = cond(G);                
end
%% Creating plots and saving values
plotConvergence(residual,iterations,condNr,Q,index)
save('ConvergenceData.mat','residual','iterations','condNr','Q','index',...
    'wsol');