%******************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @since: 03-12-2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Parameters for two component HEX
% @componentList: 1: Methanol   2: Water
%******************************************************************

function par = parameters()
%% Literature 
% https://webbook.nist.gov/cgi/cbook.cgi?ID=C67561&Mask=4&Type=ANTOINE&Plot=on
% https://webbook.nist.gov/cgi/cbook.cgi?ID=C7732185&Mask=4&Type=ANTOINE&Plot=on
% http://www.personal.utulsa.edu/~geoffrey-price/Courses/ChE7023/
% HeatCapacity-HeatOfFormation.pdf
%% Parameters
% Antoine Parameters log(p_sat[bar]) = A - B/(T[K]+C)
A = [5.15853 4.6543];
B = [1569.613 1435.264];
C = [-34.846 -64.848];

par.T_in = 0.410;       % Temperature inlet [kK]
par.F_in = 0.1;         % Inlet flow rate [kmol/s]
par.Antoine = [A;B;C];  % Antoine parameters
par.Q = 0;              % Heat transfer [MW]
par.Q_max = -4;         % Maximum heat transfer [MW]
par.h_vap = [35.210 40.660]; % Heat of vaporization [MJ/kmol]
par.Cp_V = [44.06, 35]; % Vapor Heat Capacity [MJ/kmol*kK]
par.Cp_L = [81.08, 75]; % Liquid Heat Capacity [MJ/kmol*kK]
par.R = 8.314;          % Gas constant [MJ/(kK * kmol)]
par.P_0 = 0.1;          % Outlet Pressure [MPa]
par.V_T = 0.2;          % Total volume [m^3]
par.rho = [0.792/32.04, 1/18.016]; % Liquid molar density[Mmol/m^3]
par.c_V = 1;            % Valve coefficient vapor[kmol/(bar^(-0.5) s)]
par.c_L = 5;            % Valve coefficient liquid [kmol/(bar^(-0.5) s)]
par.z = [0.5 0.5];      % Composition inlet [Hot Cold]      
par.C0 = 3*145.0377e-6; % Liquid compressability factor [1/MPa]
par.T_ref = 0.29815;    % Reference temperature [kK]
par.M = 3;              % Number of flash tanks in series

%% Calculation of inlet enthalpy [MJ/kmol] 
% This calculation assumes that the inlet is in vapor phase
par.h_in = par.z(1)*(par.h_vap(1) + par.Cp_V(1)*(par.T_in-par.T_ref))+...
        par.z(2)*(par.h_vap(2) + par.Cp_V(2)*(par.T_in-par.T_ref));
end