%******************************************************************
% @author: Marius Reed, Marlene L. Lund
% @organization: Process Systems Engineering, NTNU
% @since: September 2017
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Stores double values and their LD-derivative
%******************************************************************

classdef valder
    properties
        val  %function value
        der  %derivative value or gradient vector
    end
    methods
        %Constructor of a valder object.
        function obj = valder(a,b)
            if nargin == 0 
                obj.val = [];
                obj.der = [];
            elseif nargin == 1
                obj.val = a;
                obj.der = 0;
            else
                obj.val = a;
                obj.der = b;
            end
        end
        
        function val = getVal(obj)
            val = obj.val;
        end
        
        function der = getDer(obj)
            der = obj.der;
        end
        
        % Creating a vector from the valder object.
        function vec = double(obj)
            vec = [obj.val, obj.der];
        end
        
        % Overloading plus for the object
        function h = plus(u,v)
            if ~isa(u,'valder')
                h = valder(u+v.val, v.der);
            elseif ~isa(v,'valder')
                h = valder(v+u.val, u.der);
            else
                h = valder(u.val+v.val, u.der+v.der);
            end
        end
        
        % Overloading negative for the object
        function h = uminus(u)
            h = valder(-u.val, -u.der);
        end
        
        % Overloading minus for the object
        function h = minus(u,v)
            if ~isa(u,'valder')
                h = valder(u-v.val, -v.der);
            elseif ~isa(v,'valder')
                h = valder(u.val-v, u.der);
            else
                h = valder(u.val-v.val, u.der-v.der);
            end
        end
        
        % Overloading multiplication for the object
        function h = mtimes(u,v)
            if ~isa(u,'valder')
                h = valder(u*v.val, u*v.der);
            elseif ~isa(v,'valder')
                h = valder(v*u.val, v*u.der);
            else
                h = valder(u.val*v.val, u.der*v.val + u.val*v.der);
            end
        end
        
        % Overloading division for the object
        function h = mrdivide(u,v)
            if ~isa(u,'valder')
                h = valder(u/v.val, -u*v.der/(v.val)^2);
            elseif ~isa(v,'valder')
                h = valder(u.val/v, u.der/v);
            else
                h = valder(u.val/v.val, (u.der*v.val - u.val*v.der)/v.val^2);
            end
        end
        
        % Overloading power for the object
        function h = mpower(u,v)
            if ~isa(u,'valder')
                h = valder(u^v.val, u^v.val*log(u)*v.der);
            elseif ~isa(v,'valder')
                h = valder(u.val^v, v*u.val^(v-1)*u.der);
            else
                h = exp(v*log(u));
            end
        end
        
        % Overloading exponenital for the object
        function h = exp(u)
            h = valder(exp(u.val), exp(u.val)*u.der);
        end
        
        % Overloading log for the object
        function h = log(u)
            h = valder(log(u.val),(1/u.val)*u.der);
        end
        % Overloading log10 for the object
        function h = log10(u)
            h = valder(log10(u.val),(1/u.val)*u.der);
        end
        
        % Overloading the square root for the object
        function h = sqrt(u)
            h = valder(sqrt(u.val), u.der/(2*sqrt(u.val)));
        end
        
        % Overloading sine for the object
        function h = sin(u)
            h = valder(sin(u.val), cos(u.val)*u.der);
        end
        
        % Overloading cosine for the object
        function h = cos(u)
            h = valder(cos(u.val), -sin(u.val)*u.der);
        end
        
        % Overloading tan for the object
        function h = tan(u)
            h = valder(tan(u.val), sec(u.val)^2*u.der);
        end
        
        % Overloading arcsine for the object
        function h = asin(u)
            h = valder(asin(u.val), u.der/sqrt(1-u.val^2));
        end
        
        % Overloading arctan for the object
        function h = atan(u)
            h = valder(atan(u.val), u.der/(1+u.val^2));
        end
         
        % Overloading the absolute function for the object using 
        % lexicographic derivatives
        function u = abs(u)
            s.type = '()'; % reference type
            for i = 1:length(u.val)
                s.subs = {i};
                uvar = subsref(u,s); % store ith element
                x = double(uvar); % convert to double

                % assign abs value and derivative to output :
                v = abs(uvar.val);
                d = valder.fsign(x)*uvar.der ;
                u = subsasgn (u, s, valder(v, d));
            end
        end
        
        % Overloading the max function for the object
        function h = max(u)
            s.type = '()';
            if length(u.val) > 2
                s.subs = {length(u.val)};
                u_l = subsref(u,s);
                s.subs = {1:length(u.val)-1};
                u_f = subsref(u,s);
                h = max2(u_l,max(u_f));
            else
                s.subs = {1};
                u_f = subsref(u,s);
                s.subs = {2};
                u_l = subsref(u,s);
                h = max2(u_f,u_l);
            end
        end
        
        % Overloading the min function for the object
        function h = min(u)
            s.type = '()';
            if length(u.val) > 2
                s.subs = {length(u.val)};
                u1 = subsref(u,s);
                s.subs = {1:length(u.val)-1};
                u2 = subsref(u,s);
                h = min2(u1,min(u2));
            elseif length(u.val) == 1
                h = u;
            else
                s.subs = {1};
                u1 = subsref(u,s);
                s.subs = {2};
                u2 = subsref(u,s);
                h = min2(u1,u2);
            end
        end
        
        % Overloading the max function for two objects
        function h = max2(u,v)
            h = (u + v + abs(u-v))/2;
        end
        
        % Overloading the min function for two objects
        function h = min2(u,v)
            h = (u + v - abs(u-v))/2;
        end
        
        % Overloading the mid-function for the object
        function h = mid(u)
            s.type = '()';
            s.subs = {1};
            u1 = subsref(u,s);
            s.subs = {2};
            u2 = subsref(u,s);
            s.subs = {3};
            u3 = subsref(u,s);
            s.subs = {1};
            u = subsasgn(u,s,min2(u1,u2));
            s.subs = {2};
            u = subsasgn(u,s,min2(u1,u3));
            s.subs = {3};
            u = subsasgn(u,s,min2(u2,u3));
            h = max(u);
        end
        
        % Overloading the midfunction for the three objects
        function h = midobj(u1,u2,u3)
            s.type = '()';
            s.subs = {1};
            u = valder();
            u = subsasgn(u,s,min2(u1,u2));
            s.subs = {2};
            u = subsasgn(u,s,min2(u1,u3));
            s.subs = {3};
            u = subsasgn(u,s,min2(u2,u3));
            h = max(u);
        end
        
        % Overloading the 1 and inf norm for the object
        function h = norm(u,p)
            switch p
                case 1
                    h = sum(abs(u));
                case inf
                    h = max(abs(u));
            end
        end
        
        % Overloading the mnorm of the function
        function h = mnorm(u,p)
            nsq = length(u.val);
            n = sqrt(nsq);
            if mod(n,1) > 0
                n = nsq;
            end
            S.type = '()';
            D.type = '()';
            uabs = abs(u);
            h = double(zeros(n,1), zeros(n,length(getDer(u))));
            switch p
                case 1
                    j = 1;
                    for i = 1:nsq
                        S.subs = {i};
                        D.subs = {j};
                        h = subsasgn(h, S, subsref(h,D) + subsref(uabs, S));
                        if mod(1,n) == 0
                            j = j + 1;
                        end
                    end
                    h = max(h);
                    
                case inf
                    j = 1;
                    for i  = 1:nsq;
                        S.subs = {i};
                        D.subs = {j};
                        h = subsasgn(h,S, subsref(h,D) + subsref(uabs, S));
                        if j < n
                            j = j + 1;
                        else 
                            j = 1;
                        end
                    end
            end
        end
        % overloads indexed reference
        function h = subsref (u,S)
            SD.type = S.type;
            SD.subs = {S.subs{1} , ':'}; % get row number 1
            h = valder(subsref(u.val,S), subsref(u.der,SD));
        end
        % overloads indexed assignment
        function obj = subsasgn (obj ,S,u)
            SD. type = S.type ;
            SD. subs = {S.subs{1} ,':'}; % get row number i
            A = subsasgn (obj.val , S, u.val);
            B = subsasgn (obj.der , SD , u.der );
            obj = valder(A,B);
        end
        
        % Overloading the sum function for the object
        function h = sum(u)
            s.type = '()';
            s.subs = {1};
            h = subsref(u,s);
            if length(getVal(u)) > 1
                for i = 2:length(getVal(u))
                    s.subs = {i};
                    h = h + subsref(u,s);
                end
            end
        end
        
    end
    
    methods ( Static ) 
        % Returns sign of the first nonzero element in x
        function s = fsign (x)
            i = 1;
            while x(i) == 0 && i< length (x)
                i = i+1;
            end
            s = sign (x(i));
        end
    end
end
