%******************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @since: 03-12-2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Single component Flash tank model
% @input: t = time, w = system variables data = parameters
% @output: F = residuals, G = Generalized Derivative Matrix
%******************************************************************

function [F,G] = Flash(t,w,data)
%% Unpacking parameters
par = data.par;
F_in = par.F_in;            % Inlet flow rate [kmol/s]
Q = par.Q;                  % Heat transfer [MW]
R = par.R;                  % Gas constant [MJ/kmol*kK]
V_T = par.V_T;              % Total volume [m^3]
P_0 = par.P_0;              % Outlet pressure [MPa]
Cp_V_1 = par.Cp_V(1);       % Vapor Heat Capcity [MJ/kmol*kK]
Cp_L_1 =  par.Cp_L(1);      % Liquid Heat Capcity [MJ/kmol*kK]
h_vap_1 = par.h_vap(1);     % Heat of vaporization [MJ/kmol]
A_1 = par.Antoine(1,1);     % Antoine Parameter
B_1 = par.Antoine(2,1);     % Antoine Parameter
C_1 = par.Antoine(3,1);     % Antoine Parameter
rho_1 = par.rho(1);         % Molar density [Mmol/m^3]
z_1 = par.z(1);             % Inlet composition (Should be 1!)
h_in = par.h_in;            % Inlet molar enthalpy [MJ/kmol]
C0 = par.C0;                % Compressability factor [1/MPa]
T_ref = par.T_ref;          % Reference temperature [kK]
c_V = par.c_V;              % Vapor valve coefficient [kmol/bar^0.5*s]
c_L = par.c_L;              % Liquid valve coefficient [kmol/bar^0.5*s]


%% Unpacking system variables
w = valder(w,eye(length(w)));
M_1 = w(1);        % Component Molar hold-up [kmol]
U = w(2);          % Internal Energy [MJ]
F_V = w(3);        % Vapor flow rate [kmol/s]
F_L = w(4);        % Liquid flow rate [kmol/s]
M_V = w(5);        % Vapor molar hold-up [kmol]
P = w(6);          % Pressure [MPa]
T = w(7);          % Temperature [kK]
V_L = w(8);        % Liquid Volume [m^3]
x_1 = w(9);        % Liquid mole fraction
y_1 = w(10);       % Vapor mole fraction
H = w(11);         % Enthalpy [MJ]
h_L = w(12);       % Molar enthalpy in liquid phase [MJ/kmol]
h_V = w(13);       % Molar enthalpy in vapor phase [MJ/kmol]
V_V = w(14);       % Vapor volume [m^3]
rho_L = w(15);     % Liquid molar density [Mmol/kK]
M_L = w(16);       % Liquid molar hold-up [kmol]
k_1 = w(17);       % Vapor-liquid equilibrium coefficent 
Ps_1 = w(18);      % Saturation pressure


%% Differential equations
f1 = F_in*z_1 - F_L*x_1 - F_V * y_1;
f2 = F_in*h_in - F_L*h_L - F_V*h_V + Q;

%% Algebraic equations
f3 = M_1 - (M_L*x_1 + M_V*y_1);
f4 = M_1 - (M_L + M_V);
f5 = H - (M_L*h_L + M_V*h_V);
f6 = H - (U + P*V_T);
f7 = y_1 - k_1*x_1;
f8 = midobj(M_V/(M_V+M_L),x_1 - y_1, (M_V/(M_V+M_L))-1);
f9 = V_T - (V_L + V_V);
f10 = P*V_V - M_V*R*T;
f11 = V_L - M_L/(rho_L*1e3);
f12 = F_V*V_T - c_V*V_V*max2(0,(P-P_0)/sqrt(abs((P-P_0))+1e-10));
f13 = F_L*V_T - c_L*V_L*max2(0,(P-P_0)/sqrt(abs((P-P_0))+1e-10));
f14 = 1/rho_L - (x_1/(rho_1*(1+C0*(P-0.1))));
f15 = h_L - (x_1*Cp_L_1)*(T-T_ref);
f16 = h_V - (y_1*(h_vap_1 + Cp_V_1*(T-T_ref)));
f17 = k_1*P - Ps_1;
f18 = Ps_1 - 10^(A_1 - B_1/(T*1e3+C_1))/10;

%% Extracting the values and the derivatives from the valder objects
F = [getVal(f1);getVal(f2);getVal(f3);getVal(f4);getVal(f5);getVal(f6);...
    getVal(f7);getVal(f8);getVal(f9);getVal(f10);getVal(f11);...
    getVal(f12);getVal(f13);getVal(f14);getVal(f15);getVal(f16);...
    getVal(f17);getVal(f18)];

G = [getDer(f1);getDer(f2);getDer(f3);getDer(f4);getDer(f5);getDer(f6);...
    getDer(f7);getDer(f8);getDer(f9);getDer(f10);getDer(f11);...
    getDer(f12);getDer(f13);getDer(f14);getDer(f15);getDer(f16);...
    getDer(f17);getDer(f18)];

end