%******************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @since: 03-12-2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: One side of heat exchanger with given Q with one component
% @input: t = time, w = system variables, data = parameters
% @output: F = residuals, G = Generalized Derivative Matrix
%******************************************************************

function [F,G] = HEX(t,w,data)
%% Unpacking parameters
par = data.par;
F_in = par.F_in;            % Inlet flow rate [kmol/s]
Q = par.Q;                  % Heat transfer [MW]
R = par.R;                  % Gas constant [MJ/kmol*kK]
V_T = par.V_T;              % Total volume [m^3]
P_0 = par.P_0;              % Outlet pressure [MPa]
Cp_V_1 = par.Cp_V(1);       % Vapor Heat Capcity [MJ/kmol*kK]
Cp_L_1 =  par.Cp_L(1);      % Liquid Heat Capcity [MJ/kmol*kK]
h_vap_1 = par.h_vap(1);     % Heat of vaporization [MJ/kmol]
A_1 = par.Antoine(1,1);     % Antoine Parameter
B_1 = par.Antoine(2,1);     % Antoine Parameter
C_1 = par.Antoine(3,1);     % Antoine Parameter
rho_1 = par.rho(1);         % Molar density [Mmol/m^3]
z_1 = par.z(1);             % Inlet composition (Should be 1!)
h_in = par.h_in;            % Inlet molar enthalpy [MJ/kmol]
M = par.M;                  % Number of flash tanks in series
C0 = par.C0;                % Compressability factor [1/MPa]
T_ref = par.T_ref;          % Reference temperature [kK]
c_V = par.c_V*sqrt(M);      % Vapor valve coefficient [kmol/bar^0.5*s]
c_L = par.c_L*sqrt(M);      % Liquid valve coefficient [kmol/bar^0.5*s]

%% Unpacking system variables
L = length(w);
w = valder(w,eye(length(w)));
M_1 = w(1:M);               % Component Molar hold-up [kmol]
U = w(1*M+1:2*M);           % Internal Energy [MJ]
F_V = w(2*M+1:3*M);         % Vapor flow rate [kmol/s]
F_L = w(3*M+1:4*M);         % Liquid flow rate [kmol/s]
M_V = w(4*M+1:5*M);         % Vapor molar hold-up [kmol]
P = w(5*M+1:6*M);           % Pressure [MPa]
T = w(6*M+1:7*M);           % Temperature [kK]
V_L = w(7*M+1:8*M);         % Liquid Volume [m^3]
x_1 = w(8*M+1:9*M);         % Liquid mole fraction
y_1 = w(9*M+1:10*M);        % Vapor mole fraction
H = w(10*M+1:11*M);         % Enthalpy [MJ]
h_L = w(11*M+1:12*M);       % Molar enthalpy in liquid phase [MJ/kmol]
h_V = w(12*M+1:13*M);       % Molar enthalpy in vapor phase [MJ/kmol]
V_V = w(13*M+1:14*M);       % Vapor volume [m^3]
rho_L = w(14*M+1:15*M);     % Liquid molar density [Mmol/kK]
M_L = w(15*M+1:16*M);       % Liquid molar hold-up [kmol]
k_1 = w(16*M+1:17*M);       % Vapor-liquid equilibrium coefficent 
Ps_1 = w(17*M+1:18*M);      % Saturation pressure

%% Preallocating memory
f1 = valder(zeros(M,1),zeros(M,L));
f2 = valder(zeros(M,1),zeros(M,L));
f3 = valder(zeros(M,1),zeros(M,L));
f4 = valder(zeros(M,1),zeros(M,L));
f5 = valder(zeros(M,1),zeros(M,L));
f6 = valder(zeros(M,1),zeros(M,L));
f7 = valder(zeros(M,1),zeros(M,L));
f8 = valder(zeros(M,1),zeros(M,L));
f9 = valder(zeros(M,1),zeros(M,L));
f10 = valder(zeros(M,1),zeros(M,L));
f11 = valder(zeros(M,1),zeros(M,L));
f12 = valder(zeros(M,1),zeros(M,L));
f13 = valder(zeros(M,1),zeros(M,L));
f14 = valder(zeros(M,1),zeros(M,L));
f15 = valder(zeros(M,1),zeros(M,L));
f16 = valder(zeros(M,1),zeros(M,L));
f17 = valder(zeros(M,1),zeros(M,L));
f18 = valder(zeros(M,1),zeros(M,L));

%% Differential equations
f1(1) = F_in*z_1 - F_L(1)*x_1(1) - F_V(1) * y_1(1);
f2(1) = F_in*h_in - F_L(1)*h_L(1) - F_V(1)*h_V(1) + Q/M;
for j = 2:M
    f1(j) = F_L(j-1)*x_1(j-1) + F_V(j-1)*y_1(j-1)...
        - (F_L(j)*x_1(j) + F_V(j)*y_1(j));
    f2(j) = F_L(j-1)*h_L(j-1) + F_V(j-1)*h_V(j-1)...
        - (F_L(j)*h_L(j) + F_V(j)*h_V(j)) + Q/M;
end
%% Algebraic equations
for j = 1:M
    f3(j) = M_1(j) - (M_L(j)*x_1(j) + M_V(j)*y_1(j));
    f4(j) = (M_1(j)) - (M_L(j) + M_V(j));
    f5(j) = H(j) - (M_L(j)*h_L(j) + M_V(j)*h_V(j));
    f6(j) = H(j) - (U(j) + P(j)*V_T/M);
    f7(j) = y_1(j) - k_1(j)*x_1(j);
    f8(j) = midobj(M_V(j)/(M_V(j)+M_L(j)),x_1(j) - y_1(j),...
        (M_V(j)/(M_V(j)+M_L(j)))-1);
    f9(j) = V_T/M - (V_L(j) + V_V(j));
    f10(j) = P(j)*V_V(j) - M_V(j)*R*T(j);
    f11(j) = V_L(j) - M_L(j)/(rho_L(j)*1e3); 
    if j < M
        f12(j) = F_V(j)*V_T/M - c_V*V_V(j)*((P(j)-P(j+1))...
            /sqrt(abs((P(j)-P(j+1)))+1e-10));
        f13(j) = F_L(j)*V_T/M - c_L*V_L(j)*((P(j)-P(j+1))...
            /sqrt(abs((P(j)-P(j+1)))+1e-10));
    else
        f12(j) = F_V(j)*V_T/M - c_V*V_V(j)*max2(0,(P(j)-P_0)...
            /sqrt(abs((P(j)-P_0))+1e-10));
        f13(j) = F_L(j)*V_T/M - c_L*V_L(j)*max2(0,((P(j)-P_0)...
            /sqrt(abs((P(j)-P_0))+1e-10)));
    end
    f14(j) = rho_L(j)*(x_1(j)) - (rho_1*(1+C0*(P(j)-0.1)));
    f15(j) = h_L(j) - (x_1(j)*Cp_L_1)*(T(j)-T_ref);
    f16(j) = h_V(j) - (y_1(j)*(h_vap_1 + Cp_V_1*(T(j)-T_ref)));
    f17(j) = k_1(j)*P(j) - Ps_1(j);
    f18(j) = Ps_1(j) - 10^(A_1 - B_1/(T(j)*1e3+C_1))/10;
    
%% Extracting the values and the derivatives from the valder objects
F = [getVal(f1);getVal(f2);getVal(f3);getVal(f4);getVal(f5);getVal(f6);...
    getVal(f7);getVal(f8);getVal(f9);getVal(f10);getVal(f11);...
    getVal(f12);getVal(f13);getVal(f14);getVal(f15);getVal(f16);...
    getVal(f17);getVal(f18)];

G = [getDer(f1);getDer(f2);getDer(f3);getDer(f4);getDer(f5);getDer(f6);...
    getDer(f7);getDer(f8);getDer(f9);getDer(f10);getDer(f11);...
    getDer(f12);getDer(f13);getDer(f14);getDer(f15);getDer(f16);...
    getDer(f17);getDer(f18)];
end