%******************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @since: 03-12-2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Counter current heat exchanger
% @input: t = time, w = system variables, data = parameters
% @output: F = residuals, G = Generalized Derivative Matrix,
%          Q = Heat transfer, F_in_2 = Inlet flow rate cold Side
%          F_in = Inlet flow rate hot side,
%          T_in_2 = Inlet temperature cold side
%******************************************************************

function [F,G,Q,F_in_2,F_in,T_in_2] = HEX_CC(t,w,data)
%% Unpacking parameters
par = data.par;
F_in = par.F_in;            % Inlet flow rate on hot side [kmol/s]
F_in_2 = par.F_in_2;        % Inlet flow rate on cold side [kmol/s]
R = par.R;                  % Gas constant [MJ/kmol*kK]
V_T = par.V_T;              % Total volume [m^3]
P_0 = par.P_0;              % Outlet pressure [MPa]
Cp_V_1 = par.Cp_V(1);       % Vapor heat capacity component 1 [MJ/kmol*kK]
Cp_L_1 =  par.Cp_L(1);      % Liquid heat capacity component 1 [MJ/kmol*kK]
Cp_V_2 = par.Cp_V(2);       % Vapor heat capacity component 2 [MJ/kmol*kK]
Cp_L_2 =  par.Cp_L(2);      % Liquid heat capacity component 1 [MJ/kmol*kK]
h_vap_1 = par.h_vap(1);     % Heat of vaporization component 1 [MJ/kmol]
h_vap_2 = par.h_vap(2);     % Heat of vaporization component 2 [MJ/kmol]
A_1 = par.Antoine(1,1);     % Antoine Parameter component 1
B_1 = par.Antoine(2,1);     % Antoine Parameter component 1
C_1 = par.Antoine(3,1);     % Antoine Parameter component 1
A_2 = par.Antoine(1,2);     % Antoine Parameter component 2
B_2 = par.Antoine(2,2);     % Antoine Parameter component 2
C_2 = par.Antoine(3,2);     % Antoine Parameter component 2
rho_1 = par.rho(1);         % Molar density component 1 [Mmol/m^3]
rho_2 = par.rho(2);         % Molar density component 2 [Mmol/m^3]
z_1 = par.z(1);             % Molar fraction hot side (Should be 1)
z_2 = par.z(2);             % Molar fraction cold side (Should be 1)
h_in = par.h_in;            % Inlet molar enthalpy hot side [MJ/kmol]
h_in_2 = par.h_in_2;        % Inlet molar enthalpy hot side [MJ/kmol]
T_in_2 = par.T_in_2;        % Inlet temperature cold side [kK]
M = par.M;                  % Number of flash tanks in series
C0 = par.C0;                % Compressability factor [1/MPa]
T_ref = par.T_ref;          % Reference temperature [kK]
c_V = par.c_V*sqrt(M);      % Vapor valve coefficient [kmol/bar^.5 s]
c_L = par.c_L*sqrt(M);      % Liquid valve coefficient [kmol/bar^.5 s]
UA = data.par.UA;           % Heat transfer coefficent times area [MW/kK]

%% Unpacking hot side variables
L = length(w);
w = valder(w,eye(length(w)));
M_1 = w(1:M);               % Component Molar hold-up [kmol]
U = w(1*M+1:2*M);           % Internal Energy [MJ]
F_V = w(2*M+1:3*M);         % Vapor flow rate [kmol/s]
F_L = w(3*M+1:4*M);         % Liquid flow rate [kmol/s]
M_V = w(4*M+1:5*M);         % Vapor molar hold-up [kmol]
P = w(5*M+1:6*M);           % Pressure [MPa]
T = w(6*M+1:7*M);           % Temperature [kK]
V_L = w(7*M+1:8*M);         % Liquid Volume [m^3]
x_1 = w(8*M+1:9*M);         % Liquid mole fraction
y_1 = w(9*M+1:10*M);        % Vapor mole fraction
H = w(10*M+1:11*M);         % Enthalpy [MJ]
h_L = w(11*M+1:12*M);       % Molar enthalpy in liquid phase [MJ/kmol]
h_V = w(12*M+1:13*M);       % Molar enthalpy in vapor phase [MJ/kmol]
V_V = w(13*M+1:14*M);       % Vapor volume [m^3]
rho_L = w(14*M+1:15*M);     % Liquid molar density [Mmol/kK]
M_L = w(15*M+1:16*M);       % Liquid molar hold-up [kmol]
k_1 = w(16*M+1:17*M);       % Vapor-liquid equilibrium coefficent 
Ps_1 = w(17*M+1:18*M);      % Saturation pressure

%% Unpacking hot side variables
M_1_2 = w(18*M+1:19*M);     % Component Molar hold-up [kmol]
U_2 = w(19*M+1:20*M);       % Internal Energy [MJ]
F_V_2 = w(20*M+1:21*M);     % Vapor flow rate [kmol/s]
F_L_2 = w(21*M+1:22*M);     % Liquid flow rate [kmol/s]
M_V_2 = w(22*M+1:23*M);     % Vapor molar hold-up [kmol]
P_2 = w(23*M+1:24*M);       % Pressure [MPa]
T_2 = w(24*M+1:25*M);       % Temperature [kK]
V_L_2 = w(25*M+1:26*M);     % Liquid Volume [m^3]
x_1_2 = w(26*M+1:27*M);     % Liquid mole fraction
y_1_2 = w(27*M+1:28*M);     % Vapor mole fraction
H_2 = w(28*M+1:29*M);       % Enthalpy [MJ]
h_L_2 = w(29*M+1:30*M);     % Molar enthalpy in liquid phase [MJ/kmol]
h_V_2 = w(30*M+1:31*M);     % Molar enthalpy in vapor phase [MJ/kmol]
V_V_2 = w(31*M+1:32*M);     % Vapor volume [m^3]
rho_L_2 = w(32*M+1:33*M);   % Liquid molar density [Mmol/kK]
M_L_2 = w(33*M+1:34*M);     % Liquid molar hold-up [kmol]
k_1_2 = w(34*M+1:35*M);     % Vapor-liquid equilibrium coefficent 
Ps_1_2 = w(35*M+1:36*M);    % Saturation pressure

%% Preallocating memory
f1 = valder(zeros(M,1),zeros(M,L));
f2 = valder(zeros(M,1),zeros(M,L));
f3 = valder(zeros(M,1),zeros(M,L));
f4 = valder(zeros(M,1),zeros(M,L));
f5 = valder(zeros(M,1),zeros(M,L));
f6 = valder(zeros(M,1),zeros(M,L));
f7 = valder(zeros(M,1),zeros(M,L));
f8 = valder(zeros(M,1),zeros(M,L));
f9 = valder(zeros(M,1),zeros(M,L));
f10 = valder(zeros(M,1),zeros(M,L));
f11 = valder(zeros(M,1),zeros(M,L));
f12 = valder(zeros(M,1),zeros(M,L));
f13 = valder(zeros(M,1),zeros(M,L));
f14 = valder(zeros(M,1),zeros(M,L));
f15 = valder(zeros(M,1),zeros(M,L));
f16 = valder(zeros(M,1),zeros(M,L));
f17 = valder(zeros(M,1),zeros(M,L));
f18 = valder(zeros(M,1),zeros(M,L));
f19 = valder(zeros(M,1),zeros(M,L));
f20 = valder(zeros(M,1),zeros(M,L));
f21 = valder(zeros(M,1),zeros(M,L));
f22 = valder(zeros(M,1),zeros(M,L));
f23 = valder(zeros(M,1),zeros(M,L));
f24 = valder(zeros(M,1),zeros(M,L));
f25 = valder(zeros(M,1),zeros(M,L));
f26 = valder(zeros(M,1),zeros(M,L));
f27 = valder(zeros(M,1),zeros(M,L));
f28 = valder(zeros(M,1),zeros(M,L));
f29 = valder(zeros(M,1),zeros(M,L));
f30 = valder(zeros(M,1),zeros(M,L));
f31 = valder(zeros(M,1),zeros(M,L));
f32 = valder(zeros(M,1),zeros(M,L));
f33 = valder(zeros(M,1),zeros(M,L));
f34 = valder(zeros(M,1),zeros(M,L));
f35 = valder(zeros(M,1),zeros(M,L));
f36 = valder(zeros(M,1),zeros(M,L));

%% Defining how some of the input will change during simulation

% Changing the temperature in the inlet on the cold side
if t < 5
    T_in_2 = T_in_2;
    h_in_2 = h_in_2;
elseif t < 70
    T_in_2 = T_in_2 - 0.0003*(t-5);
    h_in_2 = par.Cp_L(2)*(T_in_2-T_ref);
else
    T_in_2 = T_in_2 - 0.0003*65;
    h_in_2 = par.Cp_L(2)*(T_in_2-T_ref);
end

% Changing the inlet flow rate on the cold side
if t < 100
    F_in_2 = F_in_2;
elseif t < 150
    F_in_2 = F_in_2 + 0.001*(t-100);
else
    F_in_2 = 0.15;
end
    
%% Differential equations
f1(1) = (F_in*z_1 - (max2(0,F_L(1))*x_1(1) +  max2(0,F_V(1))*y_1(1))...
    - (min2(0,F_L(1))*x_1(2) +  min2(0,F_V(1))*y_1(2)));
f2(1) = (F_in*h_in - (max2(0,F_L(1))*h_L(1) + max2(0,F_V(1))*h_V(1))...
    - min2(0,F_L(1))*h_L(2) + min2(0,F_V(1))*h_V(2) + UA*(T_2(M)-T(1)));
f19(1) = (F_in_2*z_2 - (max2(0,F_L_2(1))*x_1_2(1)...
    +  max2(0,F_V_2(1))*y_1_2(1)) - (min2(0,F_L_2(1))*x_1_2(2)...
    +  min2(0,F_V_2(1))*y_1_2(2)));
f20(1) = (F_in_2*h_in_2 - (max2(0,F_L_2(1))*h_L_2(1)...
    + max2(0,F_V_2(1))*h_V_2(1)) - (min2(0,F_L_2(1))*h_L_2(2)...
    + min2(0,F_V_2(1))*h_V_2(2)) + UA*(T(M)-T_2(1)));
Q(1) = getVal(UA*(T_2(M)-T(1)));
for j = 2:M-1
    f1(j) = (max2(0,F_L(j-1)*x_1(j-1) + F_V(j-1)*y_1(j-1))...
        - max2(0,F_L(j)*x_1(j) + F_V(j)*y_1(j)) + min2(0,F_L(j-1)*x_1(j)...
        + F_V(j-1)*y_1(j)) - min2(0,F_L(j)*x_1(j+1) + F_V(j)*y_1(j+1)));
    f2(j) = ((max2(0,F_L(j-1))*h_L(j-1) + max2(0,F_V(j-1))*h_V(j-1))...
        - (max2(0,F_L(j))*h_L(j) + max2(0,F_V(j))*h_V(j))...
        + (min2(0,F_L(j-1))*h_L(j) + min2(0,F_V(j-1))*h_V(j))...
        - (min2(0,F_L(j))*h_L(j+1) + min2(0,F_V(j))*h_V(j+1))...
        + UA*(T_2(M-j+1)-T(j)));
    f19(j) = (max2(0,F_L_2(j-1)*x_1_2(j-1) + F_V_2(j-1)*y_1_2(j-1))...
        - max2(0,F_L_2(j)*x_1_2(j) + F_V_2(j)*y_1_2(j))...
        + min2(0,F_L_2(j-1)*x_1_2(j) + F_V_2(j-1)*y_1_2(j))...
        - min2(0,F_L_2(j)*x_1_2(j+1) + F_V_2(j)*y_1_2(j+1)));
    f20(j) = ((max2(0,F_L_2(j-1))*h_L_2(j-1)...
        + max2(0,F_V_2(j-1))*h_V_2(j-1)) - (max2(0,F_L_2(j))*h_L_2(j)...
        + max2(0,F_V_2(j))*h_V_2(j)) + (min2(0,F_L_2(j-1))*h_L_2(j)...
        + min2(0,F_V_2(j-1))*h_V_2(j)) - (min2(0,F_L_2(j))*h_L_2(j+1)...
        + min2(0,F_V_2(j))*h_V_2(j+1))  + UA*(T(M-j+1)-T_2(j)));
    Q(j) = getVal(UA*(T_2(M-j+1)-T(j)));
end
f1(M) = (max2(0,F_L(M-1)*x_1(M-1) + F_V(M-1)*y_1(M-1))...
    - max2(0,F_L(M)*x_1(M) + F_V(M)*y_1(M))...
    + min2(0,F_L(M-1)*x_1(M) + F_V(M-1)*y_1(M)));
f2(M) = ((max2(0,F_L(M-1))*h_L(M-1) + max2(0,F_V(M-1))*h_V(M-1))...
    - (max2(0,F_L(M))*h_L(M) + max2(0,F_V(M))*h_V(M))...
    + (min2(0,F_L(M-1))*h_L(M) + min2(0,F_V(M-1))*h_V(M))...
    + UA*(T_2(1)-T(M)));
f19(M) =(max2(0,F_L_2(M-1)*x_1_2(M-1) + F_V_2(M-1)*y_1_2(M-1))...
    - max2(0,F_L_2(M)*x_1_2(M) + F_V_2(M)*y_1_2(M))...
    + min2(0,F_L_2(M-1)*x_1_2(M) + F_V_2(M-1)*y_1_2(M)));
f20(M) = ((max2(0,F_L_2(M-1))*h_L_2(M-1) + max2(0,F_V_2(M-1))*h_V_2(M-1))...
    - (max2(0,F_L_2(M))*h_L_2(M) + max2(0,F_V_2(M))*h_V_2(M))...
    + (min2(0,F_L_2(M-1))*h_L_2(M) + min2(0,F_V_2(M-1))*h_V_2(M))...
    + UA*(T(1)-T_2(M)));
Q(M) = getVal(UA*(T_2(1)-T(M)));
%% Algebraic equations
for j = 1:M
    f3(j) = M_1(j) - (M_L(j)*x_1(j) + M_V(j)*y_1(j));
    f4(j) = (M_1(j)) - (M_L(j) + M_V(j));
    f5(j) = H(j) - (M_L(j)*h_L(j) + M_V(j)*h_V(j));
    f6(j) = H(j) - (U(j) + P(j)*V_T/M);
    f7(j) = y_1(j) - k_1(j)*x_1(j);
    f8(j) = midobj(M_V(j)/(M_V(j)+M_L(j)),x_1(j) - y_1(j),...
        (M_V(j)/(M_V(j)+M_L(j)))-1);
    f9(j) = V_T/M - (V_L(j) + V_V(j));
    f10(j) = P(j)*V_V(j) - M_V(j)*R*T(j);
    f11(j) = V_L(j)*(rho_L(j)*1e3) - M_L(j); 
    if j < M
        f12(j) = F_V(j)*V_T/M - c_V*V_V(j)*((P(j)-P(j+1))...
            /sqrt(abs((P(j)-P(j+1)))+1e-10));
        f13(j) = F_L(j)*V_T/M - c_L*V_L(j)*((P(j)-P(j+1))...
            /sqrt(abs((P(j)-P(j+1)))+1e-10));
    else
        f12(j) = F_V(j)*V_T/M - c_V*V_V(j)*max2(0,(P(j)-P_0)...
            /sqrt(abs((P(j)-P_0))+1e-10));
        f13(j) = F_L(j)*V_T/M - c_L*V_L(j)*max2(0,((P(j)-P_0)...
            /sqrt(abs((P(j)-P_0))+1e-10)));
    end
    f14(j) = rho_L(j)*(x_1(j)) - (rho_1*(1+C0*(P(j)-0.1)));
    f15(j) = h_L(j) - (x_1(j)*Cp_L_1)*(T(j)-T_ref);
    f16(j) = h_V(j) - (y_1(j)*(h_vap_1 + Cp_V_1*(T(j)-T_ref)));
    f17(j) = k_1(j)*P(j) - Ps_1(j);
    f18(j) = Ps_1(j) - 10^(A_1 - B_1/(T(j)*1e3+C_1))/10;
    
    f21(j) = M_1_2(j) - (M_L_2(j)*x_1_2(j) + M_V_2(j)*y_1_2(j));
    f22(j) = (M_1_2(j)) - (M_L_2(j) + M_V_2(j));
    f23(j) = H_2(j) - (M_L_2(j)*h_L_2(j) + M_V_2(j)*h_V_2(j));
    f24(j) = H_2(j) - (U_2(j) + P_2(j)*V_T/M);
    f25(j) = y_1_2(j) - k_1_2(j)*x_1_2(j);
    f26(j) = midobj(M_V_2(j)/(M_V_2(j)+M_L_2(j)),x_1_2(j) - y_1_2(j),...
        (M_V_2(j)/(M_V_2(j)+M_L_2(j)))-1);
    f27(j) = V_T/M - (V_L_2(j) + V_V_2(j));
    f28(j) = P_2(j)*V_V_2(j) - M_V_2(j)*R*T_2(j);
    f29(j) = V_L_2(j)*(rho_L_2(j)*1e3) - M_L_2(j); 
    if j < M
        f30(j) = F_V_2(j)*V_T/M - c_V*V_V_2(j)*((P_2(j)-P_2(j+1))...
            /sqrt(abs((P_2(j)-P_2(j+1)))+1e-10));
        f31(j) = F_L_2(j)*V_T/M - c_L*V_L_2(j)*((P_2(j)-P_2(j+1))...
            /sqrt(abs((P_2(j)-P_2(j+1)))+1e-10));
    else
        f30(j) = F_V_2(j)*V_T/M - c_V*V_V_2(j)*max2(0,(P_2(j)-P_0)...
            /sqrt(abs((P_2(j)-P_0))+1e-10));
        f31(j) = F_L_2(j)*V_T/M - c_L*V_L_2(j)*max2(0,((P_2(j)-P_0)...
            /sqrt(abs((P_2(j)-P_0))+1e-10)));
    end
    
    f32(j) = rho_L_2(j)*(x_1_2(j)) - (rho_2*(1+C0*(P_2(j)-0.1)));
    f33(j) = h_L_2(j) - (x_1_2(j)*Cp_L_2)*(T_2(j)-T_ref);
    f34(j) = h_V_2(j) - (y_1_2(j)*(h_vap_2 + Cp_V_2*(T_2(j)-T_ref)));
    f35(j) = k_1_2(j)*P_2(j) - Ps_1_2(j);
    f36(j) = Ps_1_2(j) - 10^(A_2 - B_2/(T_2(j)*1e3+C_2))/10;
    
end

%% Extracting the values and the derivatives from the valder objects
F = [getVal(f1);getVal(f2);getVal(f3);getVal(f4);getVal(f5);getVal(f6);...
    getVal(f7);getVal(f8);getVal(f9);getVal(f10);getVal(f11);...
    getVal(f12);getVal(f13);getVal(f14);getVal(f15);getVal(f16);...
    getVal(f17);getVal(f18);getVal(f19);getVal(f20);getVal(f21);...
    getVal(f22);getVal(f23);getVal(f24);getVal(f25);getVal(f26);...
    getVal(f27);getVal(f28);getVal(f29);getVal(f30);getVal(f31);...
    getVal(f32);getVal(f33);getVal(f34);getVal(f35);getVal(f36)];

G = [getDer(f1);getDer(f2);getDer(f3);getDer(f4);getDer(f5);getDer(f6);...
    getDer(f7);getDer(f8);getDer(f9);getDer(f10);getDer(f11);...
    getDer(f12);getDer(f13);getDer(f14);getDer(f15);getDer(f16);...
    getDer(f17);getDer(f18);getDer(f19);getDer(f20);getDer(f21);...
    getDer(f22);getDer(f23);getDer(f24);getDer(f25);getDer(f26);...
    getDer(f27);getDer(f28);getDer(f29);getDer(f30);getDer(f31);...
    getDer(f32);getDer(f33);getDer(f34);getDer(f35);getDer(f36)];
end