%**************************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @project: Master Thesis 2018
% @since: June 2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Main script for simulating a single component coun-
%               ter current HEX.
% @estimated run time: 12 hours
%**************************************************************************
% Description of files
% @ Flash(t,w,data): Single component flash tank model
% @ HEX(t,w,data): Single component HEX model
% @ HEX_CC(t,w,data): Single component counter current HEX model
% @ HEX_CC_implicit(t,w,data,dynVar,dt): Single component counter current
%                       HEX model with discretized differential equations
% @ implicitSolverFull_CC(w,data,dt,tspan): Euler integrator for simulating
%                                           the HEX_CC_implicit model
% @ initialGuesses_gases(par): Initial guesses and boundaries for single
%                              component Flash in vapor-only region
% @ initialGuesses_liquid(par): Initial guesses and boundaries for single
%                               component Flash in liquid-only region
% @ parmeters_CC: Parameter values for counter current HEX
% @ parameters_Methanol: Parameters for HEX model with methanol
% @ parameter_water: Paramters for HEX model with water
% @ plotCC(t,w,Q,F_in_2,T_in_2): Function for plotting result from
%                                simulation of counter current HEX
% @ valder: Matlab class for calculating the function residuals and
%           Generalized derivatives.
%**************************************************************************
clear all
clc
tic
%% Variable list
% Cold side is Var 1-18, Hot Side has the same structure and is Var 19-36
% Var1:  M_1    Molar hold up of component 1               [kmol]
% Var2 : U      Internal Energy                            [MJ]
% Var3 : F_V    Vapor outlet flow                          [kmol/s]
% Var4 : F_L    Liquid outlet flow                         [kmol/s]
% Var5 : M_V    Molar vapor hold-up                        [kmol]
% Var6 : P      Pressure                                   [MPa]
% Var7 : T      Temperature                                [K]
% Var8 : V_L    Liquid volume                              [m^3]
% Var9 : x_1    Liquid fraction of component 1             [-]
% Var10: y_1    Vapor fraction of component 1              [-]
% Var11: H      Enthalpy                                   [MJ]
% Var12: h_L    Liquid molar enthalpy                      [MJ/kmol]
% Var13: h_V    Vapor molar enthalpy                       [MJ/kmol]
% Var14: V_V    Vapor volume                               [m^3]
% Var15: rho_L  Liquid molar density                       [Mmol/m^3]
% Var16: M_L    Liquid molar hold-up                       [kmol]
% Var17: K_1    Equilibrium coeffiecient for component 1   [-]
% Var18: Ps_1   Saturation pressure for component 1        [MPa]

%% Component list:
% Hot Side: Methanol       CH3OH
% Cold Side: Water          H2O

%% Solver Settings
options_lsq = optimoptions(@lsqnonlin,'Display','iter',...
     'MaxIterations',1000,'MaxFunEvals',1e10,...
     'stepTolerance',1e-20,'FunctionTolerance',1e-20,...
     'specifyObjectiveGradient', true,...
     'OptimalityTolerance',1e-20);
 options_fsolve = optimoptions(@fsolve,'Display','iter',...
     'MaxIterations',2e3,'MaxFunEvals',1e10,...
     'stepTolerance',1e-13,'FunctionTolerance',1e-13,...
     'specifyObjectiveGradient', true,...
     'OptimalityTolerance',1e-13, 'Algorithm','levenberg-marquardt');
 
%% Solving single Flash Tank with Methanol with no Heat Transfer
data.par = parameters_Methanol();            % Parameters
[w0,lb,ub] = initialGuesses_gases(data.par); % Initial guess and boundaries
data.par.Q = 0;                           % No Heat Transfer
data.par.V_T = data.par.V_T/data.par.M;   % Volume equal to one cell in HEX
lb(14) = data.par.V_T;                    % Lower Vapor Volume Boundary
ub(14) = data.par.V_T;                    % Upper Vapor  Volume Boundary
w0_methanol = lsqnonlin(@(w) Flash(1,w,data),...
    w0,lb,ub,options_lsq);                % Solving the Flash
%% Solving a Full Single Side of HEX with no Heat Transfer
data.par = parameters_Methanol();        % Parameters
w_initial = [];
for i = 1:18                             % Repeating each variable M times
    w_initial = [w_initial;repmat(w0_methanol(i),data.par.M,1)];
end
w_initial_methanol = fsolve(@(w)HEX(1,w,data),...
    w_initial,options_fsolve);           % Solving one Side HEX
%% Solving single Flash Tank with Methanol with no Heat Transfer
data.par = parameters_water();               % Parameters
[w0,lb,ub] = initialGuesses_liquid(data.par);% Initial guess and boundaries
data.par.Q = 0;                              % No Heat Transfer
data.par.V_T = data.par.V_T/data.par.M;  % Volume equal to one cell in HEX
lb(8) = data.par.V_T;                    % Lower Liquid Volume Boundary
ub(8) = data.par.V_T;                    % Upper Liquid Volume Boundary
w0_water = lsqnonlin(@(w) Flash(1,w,data),...
    w0,lb,ub,options_lsq);               % Solving the Flash

%% Solving a Full Single Side of HEX with no Heat Transfer
data.par = parameters_water();           % Parameters
w_initial = [];                          % Clearing Var
for i = 1:18                             % Repeating each Var M Times
    w_initial = [w_initial;repmat(w0_water(i),data.par.M,1)];
end
% Solving one Side HEX
w_initial_water = fsolve(@(w)HEX(1,w,data),w_initial,options_fsolve);

%% Finding inital values with desired UA (Here UA = 4 is used)
data.par = parameters_CC();                      % Parameters
w_initial = [w_initial_methanol;w_initial_water];% Concatenating Init. Val.
%%
UA = 0:0.1:4;                                    % UA values
data.par.UA = 0;                                 % First UA Value
% Solving CC HEX with UA = 0
w_initial_Full = fsolve(@(w)HEX_CC(1,w,data),...
    w_initial,options_fsolve);
% Stepwise changing UA until desired UA
%%
for i = 2:length(UA)
    data.par.UA = UA(i);
    w_initial_Full(:,i) = fsolve(@(w)HEX_CC(1,w,data),...
        w_initial_Full(:,i-1),options_fsolve);
end

%% Simulating the CC HEX
data.par = parameters_CC();             % Setting the parameters
[t_full,w_full,Q,F_in_2,F_in,T_in_2] = implicitSolverFull_CC(...
    w_initial_Full(:,end),data,0.05,[0 200]);
% Saving the result into a .mat file
save('HEX_CC_UA_4.mat','t_full','w_full','Q','F_in_2','F_in','T_in_2','data');
%% Generating Plots
plotCC(t_full,w_full,Q,F_in_2,T_in_2)
toc