%******************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @since: 03-12-2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: This function produces plots from results obtain by
%               simulating a single component heat exchanger where 
%               both sides are divided into M = 3 flash tanks.
% @input:		t = time, w = system variables, Q = heat transfer,
%				F_in_2 = Inlet flow rate on cold side,
%				T_in_2 = Inlet temperature on cold side.
%******************************************************************


function plotCC(t,w,Q,F_in_2,T_in_2)

%% Number of Flash Tanks on Each Side
M = length(w(:,1))/36;
%% Unpacking Hot Side Variables
M_1 = w(1:M,:);
U = w(1*M+1:2*M,:);
F_V = w(2*M+1:3*M,:);
F_L = w(3*M+1:4*M,:);
M_V = w(4*M+1:5*M,:);
P = w(5*M+1:6*M,:)*10;
T = w(6*M+1:7*M,:)*1000;
V_L = w(7*M+1:8*M,:);
x_1 = w(8*M+1:9*M,:);
y_1 = w(9*M+1:10*M,:);
H = w(10*M+1:11*M,:);
h_L = w(11*M+1:12*M,:);
h_V = w(12*M+1:13*M,:);
V_V = w(13*M+1:14*M,:);
rho_L = w(14*M+1:15*M,:);
M_L = w(15*M+1:16*M,:);
k_1 = w(16*M+1:17*M,:);
Ps_1 = w(17*M+1:18*M,:);

%% Unpacking Cold Side Variables
M_1_2 = w(18*M+1:19*M,:);
U_2 = w(19*M+1:20*M,:);
F_V_2 = w(20*M+1:21*M,:);
F_L_2 = w(21*M+1:22*M,:);
M_V_2 = w(22*M+1:23*M,:);
P_2 = w(23*M+1:24*M,:)*10;
T_2 = w(24*M+1:25*M,:)*1000;
V_L_2 = w(25*M+1:26*M,:);
x_1_2 = w(26*M+1:27*M,:);
y_1_2 = w(27*M+1:28*M,:);
H_2 = w(28*M+1:29*M,:);
h_L_2 = w(29*M+1:30*M,:);
h_V_2 = w(30*M+1:31*M,:);
V_V_2 = w(31*M+1:32*M,:);
rho_L_2 = w(32*M+1:33*M,:);
M_L_2 = w(33*M+1:34*M,:);
k_1_2 = w(34*M+1:35*M,:);
Ps_1_2 = w(35*M+1:36*M,:);

%% Plot Settings
markerSet = {'-','--',':','-','--',':'};
ColorSet = [0 0.4470 0.7410;0.8500 0.3250 0.0980];
set(0,'defaulttextInterpreter','latex','DefaultAxesFontName',...
 'Times New Roman','DefaultTextFontname', 'Times New Roman',...
 'DefaultLineLineWidth',6)


%% Temperature
figure(1)
hold on
plot(t, T(1,:),'r',t,T_2(3,:),'b');
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([320 380])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('T^H_1','T^C_3','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 55)
ylabel('T [K]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Temperature
figure(2)
hold on
plot(t, T(2,:),'r',t,T_2(2,:),'b');
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([300 360])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('T^H_2','T^C_2','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 55)
ylabel('T [K]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Temperature
figure(3)
hold on
plot(t, T(3,:),'r',t,T_2(1,:),'b');
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([290 342])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('T^H_3','T^C_1','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 55)
ylabel('T [K]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Heat transfer
figure(4)
hold on
plot(t, Q(1,:));
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([-0.2, -0.1])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 55)
ylabel('$Q_{1,3}$ [MW]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Heat Transfer
figure(5)
hold on
plot(t, Q(2,:));
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([-0.2, -0.1])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 55)
ylabel('$Q_{2,2}$ [MW]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Heat Transfer
figure(6)
hold on
plot(t, Q(3,:));
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([-0.2, -0.1])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 55)
ylabel('$Q_{3,1}$ [MW]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Pressure on Cold Side
figure(7)
hold on
plot(t, P_2);
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([1, 1.01])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 55)
ylabel('$p$ [bar]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Liquid Flow Rate Cold Side
figure(8)
hold on
plot(t, F_L_2);
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([0.09, 0.16])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 55)
ylabel('$F_L$ [kmol/s]', 'interpreter','latex')
set(gca,'FontSize',55);
legend('F_{L,1}^C','F_{L,2}^C','F_{L,3}^C','Location','NorthEastOutside')

%% Tempearature in Inlet on Cold Side
figure(9)
hold on
plot(t, T_in_2*1000);
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 55)
ylabel('$T^C_{in}$ [K]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Inlet Flow Rate on Cold Side
figure(10)
hold on
plot(t, F_in_2);
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 55)
ylabel('$F^C_{in}$ [kmol/s]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Liquid Molar Hold-Up in Last Part of HEX on Cold Side
figure(11)
hold on
plot(t, M_L(3,:));
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([0, 0.02])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 55)
ylabel('$M_{L,3}^H$ [kmol]', 'interpreter','latex')
set(gca,'FontSize',55);

%% Heat Transfer
figure(12)
hold on
plot(t, sum(Q));
plot([5,5],[-1000 1000],'k--',[70,70],[-1000 1000],'k--','lineWidth',2)
plot([100,100],[-1000 1000],'g--',...
    [150,150],[-1000 1000],'g--','lineWidth',2)
ylim([-0.5, -0.3])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 40)
ylabel('$Q_{tot}$ [MW]', 'interpreter','latex')
set(gca,'FontSize',40);


end