%******************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @since: 03-12-2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Implicit two component HEX model
% @input: t = time, w = system variables data = parameters, dynVar = 
% Value of differential variables at previous time, dt = time step
% @output: F = residuals, G = Generalized Derivative Matrix
%******************************************************************

function [F,G] = HEX_implicit(t,w,data,dynVar,dt)

%% Unpacking parameters
par = data.par;
F_in = par.F_in;            % Inlet flow rate [kmol/s]
Q_0 = par.Q;                % Heat transfer [MW]
Q_max = par.Q_max;          % Maximum Heat transfer [MW]
R = par.R;                  % Gas constant [MJ/kmol*kK]
V_T = par.V_T;              % Total volume [m^3]
P_0 = par.P_0;              % Outlet pressure [MPa]
Cp_V_1 = par.Cp_V(1);       % Vapor Heat Capcity comp 1 [MJ/kmol*kK]
Cp_L_1 =  par.Cp_L(1);      % Liquid Heat Capcity comp 1 [MJ/kmol*kK]
Cp_V_2 = par.Cp_V(2);       % Vapor Heat Capcity comp 2 [MJ/kmol*kK]
Cp_L_2 =  par.Cp_L(2);      % Liquid Heat Capcity comp 2 [MJ/kmol*kK]
h_vap_1 = par.h_vap(1);     % Heat of vaporization comp 1 [MJ/kmol]
h_vap_2 = par.h_vap(2);     % Heat of vaporization comp 2 [MJ/kmol]
A_1 = par.Antoine(1,1);     % Antoine Parameter comp 1
B_1 = par.Antoine(2,1);     % Antoine Parameter comp 1
C_1 = par.Antoine(3,1);     % Antoine Parameter comp 1
A_2 = par.Antoine(1,2);     % Antoine Parameter comp 2
B_2 = par.Antoine(2,2);     % Antoine Parameter comp 2
C_2 = par.Antoine(3,2);     % Antoine Parameter comp 2
rho_1 = par.rho(1);         % Molar density comp 1 [Mmol/m^3]
rho_2 = par.rho(1);         % Molar density comp 2 [Mmol/m^3]
z_1 = par.z(1);             % Inlet composition comp 1
z_2 = par.z(2);             % Inlet composition comp 1
h_in = par.h_in;            % Inlet molar enthalpy [MJ/kmol]
C0 = par.C0;                % Compressability factor [1/MPa]
T_ref = par.T_ref;          % Reference temperature [kK]
c_V = par.c_V;              % Vapor valve coefficient [kmol/bar^0.5*s]
c_L = par.c_L;              % Liquid valve coefficient [kmol/bar^0.5*s]
M = par.M;                  % Number of flash tanks in series
% Adjusting the valve coeffcient to M
c_V = sqrt(M)*c_V;
c_L = sqrt(M)*c_L;

% Values of the differential variables at previous time.
M_1_0 = dynVar(1:M);
M_2_0 = dynVar(1*M+1:2*M);
U_0 = dynVar(2*M+1:3*M);

% Defining the changes in heat transfer 
if t < 5
    Q = Q_0;
elseif t < 100
    Q = Q_0 + (Q_max-Q_0)/(100-5)*(t-5);
else
    Q = Q_max;
end
%% Unpacking system variables
L = length(w);
w = valder(w,eye(length(w)));
M_1 = w(1:M);            % Component 1 Molar hold-up [kmol]
M_2 = w(1*M+1:2*M);      % Component 2 Molar hold-up [kmol]
U = w(2*M+1:3*M);        % Internal Energy [MJ]
F_V = w(3*M+1:4*M);      % Vapor flow rate [kmol/s]
F_L = w(4*M+1:5*M);      % Liquid flow rate [kmol/s]
M_V = w(5*M+1:6*M);      % Vapor molar hold-up [kmol]
P = w(6*M+1:7*M);        % Pressure [MPa]
T = w(7*M+1:8*M);        % Temperature [kK]
V_L = w(8*M+1:9*M);      % Liquid Volume [m^3]
x_1 = w(9*M+1:10*M);     % Liquid mole fraction component 1
x_2 = w(10*M+1:11*M);    % Liquid mole fraction component 2
y_1 = w(11*M+1:12*M);    % Vapor mole fraction component 1
y_2 = w(12*M+1:13*M);    % Vapor mole fraction component 2
H = w(13*M+1:14*M);      % Enthalpy [MJ]
h_L = w(14*M+1:15*M);    % Molar enthalpy in liquid phase [MJ/kmol]
h_V = w(15*M+1:16*M);    % Molar enthalpy in vapor phase [MJ/kmol]
V_V = w(16*M+1:17*M);    % Vapor volume [m^3]
rho_L = w(17*M+1:18*M);  % Liquid molar density [Mmol/kK]
M_L = w(18*M+1:19*M);    % Liquid molar hold-up [kmol]
k_1 = w(19*M+1:20*M);    % Vapor-liquid equilibrium coefficent component 1
k_2 = w(20*M+1:21*M);    % Vapor-liquid equilibrium coefficent component 2
Ps_1 = w(21*M+1:22*M);   % Saturation pressure component 1 
Ps_2 = w(22*M+1:23*M);   % Saturation pressure component 2

%% Preallocating memory
f1 = valder(zeros(M,1),zeros(M,L));
f2 = valder(zeros(M,1),zeros(M,L));
f3 = valder(zeros(M,1),zeros(M,L));
f4 = valder(zeros(M,1),zeros(M,L));
f5 = valder(zeros(M,1),zeros(M,L));
f6 = valder(zeros(M,1),zeros(M,L));
f7 = valder(zeros(M,1),zeros(M,L));
f8 = valder(zeros(M,1),zeros(M,L));
f9 = valder(zeros(M,1),zeros(M,L));
f10 = valder(zeros(M,1),zeros(M,L));
f11 = valder(zeros(M,1),zeros(M,L));
f12 = valder(zeros(M,1),zeros(M,L));
f13 = valder(zeros(M,1),zeros(M,L));
f14 = valder(zeros(M,1),zeros(M,L));
f15 = valder(zeros(M,1),zeros(M,L));
f16 = valder(zeros(M,1),zeros(M,L));
f17 = valder(zeros(M,1),zeros(M,L));
f18 = valder(zeros(M,1),zeros(M,L));
f19 = valder(zeros(M,1),zeros(M,L));
f20 = valder(zeros(M,1),zeros(M,L));
f21 = valder(zeros(M,1),zeros(M,L));
f22 = valder(zeros(M,1),zeros(M,L));
f23 = valder(zeros(M,1),zeros(M,L));
f24 = valder(zeros(M,1),zeros(M,L));
f25 = valder(zeros(M,1),zeros(M,L));
f26 = valder(zeros(M,1),zeros(M,L));
f27 = valder(zeros(M,1),zeros(M,L));

%% Differential equations
%% Differential equations
% f1(1) = M_1(1) - (M_1_0(1)  + (F_in*z_1 - (F_L(1)*x_1(1)...
%     + F_V(1)*y_1(1)))*dt);
% f2(1) = M_2(1) - (M_2_0(1)  + (F_in*z_2 - (F_L(1)*x_2(1)...
%     +  F_V(1)*y_2(1)))*dt);
% f3(1) = U(1) - (U_0(1)  + (F_in*h_in - (F_L(1)*h_L(1)...
%     + F_V(1)*h_V(1)) + Q/M)*dt);
% 
% for j = 2:M-1
%     f1(j) = M_1(j) - (M_1_0(j) + ((F_L(j-1)*x_1(j-1)...
%         + F_V(j-1)*y_1(j-1)) - (F_L(j)*x_1(j) + F_V(j)*y_1(j)))*dt);
%     f2(j) = M_2(j) - (M_2_0(j) + ((F_L(j-1)*x_2(j-1)...
%         + F_V(j-1)*y_2(j-1)) - (F_L(j)*x_2(j) + F_V(j)*y_2(j)))*dt);
%     f3(j) = U(j) - (U_0(j)  + ((F_L(j-1)*h_L(j-1)...
%         + F_V(j-1)*h_V(j-1)) - (F_L(j)*h_L(j) + F_V(j)*h_V(j)) + Q/M)*dt);
% end
% 
% f1(M) = M_1(M) - (M_1_0(M) + ((F_L(M-1)*x_1(M-1) + F_V(M-1)*y_1(M-1))...
%     - (F_L(M)*x_1(M) + F_V(M)*y_1(M)))*dt);
% f2(M) = M_2(M) - (M_2_0(M) + ((F_L(M-1)*x_2(M-1) + F_V(M-1)*y_2(M-1))...
%     - (F_L(M)*x_2(M) + F_V(M)*y_2(M)))*dt);
% f3(M) = U(M) - (U_0(M) + ((F_L(M-1)*h_L(M-1) + F_V(M-1)*h_V(M-1))...
%     - (F_L(M)*h_L(M) + F_V(M)*h_V(M)) + Q/M)*dt);

f1(1) = M_1(1)  - (M_1_0(1)  + (F_in*z_1 - (max2(0,F_L(1))*x_1(1)...
    +  max2(0,F_V(1))*y_1(1)) - (min2(0,F_L(1))*x_1(2) +...
    min2(0,F_V(1))*y_1(2)))*dt);
f2(1) = M_2(1)  - (M_2_0(1)  + (F_in*z_2 - (max2(0,F_L(1))*x_2(1)...
    +  max2(0,F_V(1))*y_2(1)) - (min2(0,F_L(1))*x_2(2)...
    +  min2(0,F_V(1))*y_2(2)))*dt);
f3(1) = U(1) - (U_0(1) + (F_in*h_in - (max2(0,F_L(1))*h_L(1)...
    + max2(0,F_V(1))*h_V(1)) - (min2(0,F_L(1))*h_L(2)...
    + min2(0,F_V(1))*h_V(2)) + Q/M)*dt);

for j = 2:M-1
    f1(j) = M_1(j) - (M_1_0(j) + ((max2(0,F_L(j-1))*x_1(j-1)...
        + max2(0,F_V(j-1))*y_1(j-1)) - (max2(0,F_L(j))*x_1(j)...
        + max2(0,F_V(j))*y_1(j)) + (min2(0,F_L(j-1))*x_1(j)...
        + min2(0,F_V(j-1))*y_1(j)) - (min2(0,F_L(j))*x_1(j+1)...
        + min2(0,F_V(j))*y_1(j+1)))*dt);
    f2(j) = M_2(j) - (M_2_0(j) + ((max2(0,F_L(j-1))*x_2(j-1)...
        + max2(0,F_V(j-1))*y_2(j-1)) - (max2(0,F_L(j))*x_2(j)...
        + max2(0,F_V(j))*y_2(j)) + (min2(0,F_L(j-1))*x_2(j)...
        +  min2(0,F_V(j-1))*y_2(j)) - (min2(0,F_L(j))*x_2(j+1)...
        + min2(0,F_V(j))*y_2(j+1)))*dt);
    f3(j) = U(j) - (U_0(j) + ((max2(0,F_L(j-1))*h_L(j-1)...
        + max2(0,F_V(j-1))*h_V(j-1)) - (max2(0,F_L(j))*h_L(j)...
        + max2(0,F_V(j))*h_V(j)) + (min2(0,F_L(j-1))*h_L(j)...
        + min2(0,F_V(j-1))*h_V(j)) - (min2(0,F_L(j))*h_L(j+1)...
        + min2(0,F_V(j))*h_V(j+1))  + Q/M)*dt);
end

f1(M) = M_1(M) - (M_1_0(M) + ((max2(0,F_L(M-1))*x_1(M-1)...
    + max2(0,F_V(M-1))*y_1(M-1)) - (max2(0,F_L(M))*x_1(M)...
    + max2(0,F_V(M))*y_1(M)) + (min2(0,F_L(M-1))*x_1(M)...
    + min2(0,F_V(M-1))*y_1(M)))*dt);
f2(M) = M_2(M) - (M_2_0(M) + ((max2(0,F_L(M-1))*x_2(M-1)...
    + max2(0,F_V(M-1))*y_2(M-1)) - (max2(0,F_L(M))*x_2(M)...
    + max2(0,F_V(M))*y_2(M)) + (min2(0,F_L(M-1))*x_2(M)...
    + min2(0,F_V(M-1))*y_2(M)))*dt);
f3(M) = U(M) - (U_0(M) + ((max2(0,F_L(M-1))*h_L(M-1)...
    + max2(0,F_V(M-1))*h_V(M-1)) - (max2(0,F_L(M))*h_L(M)...
    + max2(0,F_V(M))*h_V(M)) + (min2(0,F_L(M-1))*h_L(M)...
    + min2(0,F_V(M-1))*h_V(M))  + Q/M)*dt);

%% Algebraic equations
for j = 1:M
    f4(j) = M_1(j) - (M_L(j)*x_1(j) + M_V(j)*y_1(j));
    f5(j) = M_2(j) - (M_L(j)*x_2(j) + M_V(j)*y_2(j));
    f6(j) = (M_1(j)+M_2(j)) - (M_L(j) + M_V(j));
    f7(j) = H(j) - (M_L(j)*h_L(j) + M_V(j)*h_V(j));
    f8(j) = H(j) - (U(j) + P(j)*V_T/M);
    f9(j) = y_1(j) - k_1(j)*x_1(j);
    f10(j) = y_2(j) - k_2(j)*x_2(j);
    f11(j) = midobj(M_V(j)/(M_V(j)+M_L(j)),...
        ((x_1(j)+x_2(j)) - (y_1(j)+y_2(j))), (M_V(j)/(M_V(j)+M_L(j)))-1);
    f12(j) = V_T/M - (V_L(j) + V_V(j));
    f13(j) = P(j)*V_V(j) - M_V(j)*R*T(j);
    f14(j) = V_L(j) - M_L(j)/(rho_L(j)*1e3); 
    if j < M
        f15(j) = F_V(j)*V_T/M - c_V*V_V(j)*(P(j)-P(j+1))...
            /sqrt(abs((P(j)-P(j+1)))+1e-10);
        f16(j) = F_L(j)*V_T/M - c_L*V_L(j)*(P(j)-P(j+1))...
            /sqrt(abs((P(j)-P(j+1)))+1e-10);
    else
        f15(j) = F_V(j)*V_T/M - c_V*V_V(j)*max2(0,(P(j)-P_0)...
            /sqrt(abs((P(j)-P_0))+1e-10));
        f16(j) = F_L(j)*V_T/M - c_L*V_L(j)*max2(0,((P(j)-P_0)...
            /sqrt(abs((P(j)-P_0))+1e-10)));
    end
    f17(j) = 1/rho_L(j) - (x_1(j)/(rho_1*(1+C0*(P(j)-0.1)))...
        +x_2(j)/(rho_2*(1+C0*(P(j)-0.1))));
    f18(j) = h_L(j) - (x_1(j)*Cp_L_1*(T(j)-T_ref)...
        + x_2(j)*Cp_L_2*(T(j)-T_ref));
    f19(j) = h_V(j) - (y_1(j)*(h_vap_1 + Cp_V_1*(T(j)-T_ref))...
        +y_2(j)*(h_vap_2 + Cp_V_2*(T(j)-T_ref)));
    f20(j) = k_1(j)*P(j) - Ps_1(j);
    f21(j) = k_2(j)*P(j) - Ps_2(j);
    f22(j) = Ps_1(j) - 10^(A_1 - B_1/(T(j)*1e3+C_1))/10;
    f23(j) = Ps_2(j) - 10^(A_2 - B_2/(T(j)*1e3+C_2))/10;
    f24(j) = abs(V_L(j)) - V_L(j);
    f25(j) = abs(V_V(j)) - V_V(j);
    f26(j) = abs(M_L(j)) - M_L(j);
    f27(j) = abs(M_V(j)) - M_V(j);
end
    
%% Extracting the values and the derivatives from the valder objects
F = [getVal(f1);getVal(f2);getVal(f3);getVal(f4);getVal(f5);getVal(f6);...
    getVal(f7);getVal(f8);getVal(f9);getVal(f10);getVal(f11);...
    getVal(f12);getVal(f13);getVal(f14);getVal(f15);getVal(f16);...
    getVal(f17);getVal(f18);getVal(f19);getVal(f20);getVal(f21);...
    getVal(f22);getVal(f23)];

G = [getDer(f1);getDer(f2);getDer(f3);getDer(f4);getDer(f5);getDer(f6);...
    getDer(f7);getDer(f8);getDer(f9);getDer(f10);getDer(f11);...
    getDer(f12);getDer(f13);getDer(f14);getDer(f15);getDer(f16);...
    getDer(f17);getDer(f18);getDer(f19);getDer(f20);getDer(f21);...
    getDer(f22);getDer(f23)];
end