%**************************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @project: Master Thesis 2018
% @since: June 2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Main script for simulating a two component HEX
% @estimated run time: 10 hours
%**************************************************************************
% Description of files
% @ Flash(t,w,data): Two component flash tank model
% @ HEX(t,w,data): Two component HEX model
% @ HEX_implicit(t,w,data): Two component HEX model with discretized diff-
%                           erential equaionts
% @ implicitSolver_Full(w,data,dt,tspan): Euler integrator for simulating 
%                                         the two component HEX
% @ initialGuesses_gases(par): Initial guesses and boundaries for two
%                              component HEX in vapor-only region
% @ parmeters: Parameter values two component HEX with water and
%              methanol
% @ plotHEX(t,w,Q): Function for plotting result from simulation
%                   of two component HEX
% @ valder: Matlab class for calculating the function residuals and
%           Generalized derivatives.
%**************************************************************************
%**************************************************************************
clear all
clc
tic
%% Variable list
% Var1: M_1    Molar hold up of component 1 (Methanol)     [kmol]
% Var2: M_2    Molar hold up of component 1 (Water)        [kmol]
% Var3 : U      Internal Energy                            [MJ]
% Var4 : F_V    Vapor outlet flow                          [kmol/s]
% Var5 : F_L    Liquid outlet flow                         [kmol/s]
% Var6 : M_V    Molar vapor hold-up                        [kmol]
% Var7 : P      Pressure                                   [MPa]
% Var8 : T      Temperature                                [K]
% Var9 : V_L    Liquid volume                              [m^3]
% Var10: x_1    Liquid fraction of component 1 (Methanol)  [-]
% Var11: x_2    Liquid fraction of component 2 (Water)     [-]
% Var12: y_1    Vapor fraction of component 1 (Methanol)   [-]
% Var13: y_2    Vapor fraction of component 1 (Water)      [-]
% Var14: H      Enthalpy                                   [MJ]
% Var15: h_L    Liquid molar enthalpy                      [MJ/kmol]
% Var16: h_V    Vapor molar enthalpy                       [MJ/kmol]
% Var17: V_V    Vapor volume                               [m^3]
% Var18: rho_L  Liquid molar density                       [Mmol/m^3]
% Var19: M_L    Liquid molar hold-up                       [kmol]
% Var20: K_1    Equilibrium coeffiecient for component 1   [-]
% Var21: K_2    Equilibrium coeffiecient for component 2   [-]
% Var22: Ps_1   Saturation pressure for component 1        [MPa]
% Var23: Ps_2   Saturation pressure for component 2        [MPa]

% Component list:
% 1: Methanol       CH3OH
% 2: Water          H2O

%% Parameters and initial guesses
data.par = parameters();
[w0,lb,ub] = initialGuesses_gases(data.par);

%% Solver settings
options_lsq = optimoptions(@lsqnonlin,'Display','iter',...
     'MaxIterations',500,'MaxFunEvals',1e10,...
     'stepTolerance',1e-20,'FunctionTolerance',1e-20,...
     'specifyObjectiveGradient', true,...
     'OptimalityTolerance',1e-20);
options_fsolve = optimoptions(@fsolve,'Display','iter',...
     'MaxIterations',3e3,'MaxFunEvals',1e10,...
     'stepTolerance',1e-13,'FunctionTolerance',1e-6,...
     'specifyObjectiveGradient', true,...
     'OptimalityTolerance',1e-13, 'Algorithm','levenberg-marquardt');
%% Solving a two component flash tank with no heat transfer
data.par.Q = 0;
data.par.V_T = data.par.V_T/data.par.M; % Volume of Flash tank is equal to 
                                        % 1/M of the HEX
lb(17) = data.par.V_T;                  % Lower boundary on volume
ub(17) = data.par.V_T;                  % Upper boundary on volume
w0 = lsqnonlin(@(w) Flash(1,w,data), w0,lb,ub,options_lsq);
%% Full heat exchanger steady state
data.par = parameters();
w_initial = [];
% Repeating the system variables M times
for i = 1:23
    w_initial = [w_initial;repmat(w0(i),data.par.M,1)];
end
%% Increasing heat removal until the desired initial value
Q = 0:-0.01:-0.08;
data.par.Q = 0;
w_initial(:,1) = fsolve(@(w)HEX(0,w,data)...
        ,w_initial,options_fsolve);
for i = 2:length(Q)
    data.par.Q = Q(i);
    w_initial(:,i) = fsolve(@(w) HEX(0,w,data)...
        ,w_initial(:,i-1),options_fsolve);
end
%% Simulating the HEX model
[t,w,Q] = implicitSolverFull(w_initial(:,end),data,0.05,[0 200]);
%%
save('HEX_.mat','t','w','Q'); % Saving results into .mat file
plotHEX(t,w,Q);
toc