%**************************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @project: Master Thesis 2018
% @since: June 2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: Main script for simulating a two component flash tank
% @estimated run time: 30 min
%**************************************************************************
% Description of files
% @ Flash(t,w,data): Two component flash tank model
% @ Flash_implicit(t,w,data): Two component flash tank model with discret-
%                             ized differential equations
% @ implicitSolver(w,data,dt,tspan): Euler integrator for simulating the
%                                    two component flash tank
% @ initialGuesses_gases(par): Initial guesses and boundaries for two
%                              component Flash in vapor-only region
% @ parmeters: Parameter values two component flash with water and
%              methanol
% @ plotTwoComponent(t,w,Q): Function for plotting result from simulation
%                            of two component flash
% @ valder: Matlab class for calculating the function residuals and
%           Generalized derivatives.
%**************************************************************************
clear all
clc
tic
%% Variable list
% Var1: M_1    Molar hold up of component 1 (Methanol)     [kmol]
% Var2: M_2    Molar hold up of component 1 (Water)        [kmol]
% Var3 : U      Internal Energy                            [MJ]
% Var4 : F_V    Vapor outlet flow                          [kmol/s]
% Var5 : F_L    Liquid outlet flow                         [kmol/s]
% Var6 : M_V    Molar vapor hold-up                        [kmol]
% Var7 : P      Pressure                                   [MPa]
% Var8 : T      Temperature                                [K]
% Var9 : V_L    Liquid volume                              [m^3]
% Var10: x_1    Liquid fraction of component 1 (Methanol)  [-]
% Var11: x_2    Liquid fraction of component 2 (Water)     [-]
% Var12: y_1    Vapor fraction of component 1 (Methanol)   [-]
% Var13: y_2    Vapor fraction of component 1 (Water)      [-]
% Var14: H      Enthalpy                                   [MJ]
% Var15: h_L    Liquid molar enthalpy                      [MJ/kmol]
% Var16: h_V    Vapor molar enthalpy                       [MJ/kmol]
% Var17: V_V    Vapor volume                               [m^3]
% Var18: rho_L  Liquid molar density                       [Mmol/m^3]
% Var19: M_L    Liquid molar hold-up                       [kmol]
% Var20: K_1    Equilibrium coeffiecient for component 1   [-]
% Var21: K_2    Equilibrium coeffiecient for component 2   [-]
% Var22: Ps_1   Saturation pressure for component 1        [MPa]
% Var23: Ps_2   Saturation pressure for component 2        [MPa]

% Component list:
% 1: Methanol       CH3OH
% 2: Water          H2O

%% Parameters and initial guesses
data.par = parameters();
[w0,lb,ub] = initialGuesses_gases(data.par);

%% Solver settings
options_lsq = optimoptions(@lsqnonlin,'Display','iter',...
     'MaxIterations',500,'MaxFunEvals',1e10,...
     'stepTolerance',1e-20,'FunctionTolerance',1e-20,...
     'specifyObjectiveGradient', true,...
     'OptimalityTolerance',1e-20);
 
%% Solving for initial values
w0 = lsqnonlin(@(w) Flash(0,w,data), w0,lb,ub,options_lsq);

%% Simulating two component flash tank
[t,w,Q] = implicitSolver(w0,data,0.1,[0 200]);

%% Saving results and producing plots
save('Flash.mat','t','w','Q');
plotTwoComponent(t,w,Q)
