%******************************************************************
% @author: Marius Reed
% @organization: Process Systems Engineering, NTNU
% @since: 03-12-2018
% @requires: MATLAB R2017b (not tested in other releases)
% @description: This function produces plots from results obtained by
%               simulating a two component flash tank.
% @input:		t = time, w = system variables, Q = heat transfer,
%******************************************************************

function plotTwoComponent(t,w,Q)

M_1 = w(1,:);
M_2 = w(2,:);
F_V = w(4,:);
F_L = w(5,:);
M_V = w(6,:);
P = w(7,:);
T = w(8,:);
V_L = w(9,:);
x_1 = w(10,:);
x_2 = w(11,:);
y_1 = w(12,:);
y_2 = w(13,:);
V_V = w(17,:);
M_L = w(19,:);

markerSet = {'-','--',':','-','--',':'};
ColorSet = [0 0.4470 0.7410;0.8500 0.3250 0.0980];
set(0,'defaulttextInterpreter','latex','DefaultAxesFontName',...
 'Times New Roman','DefaultTextFontname', 'Times New Roman','DefaultLineLineWidth',6)

%% Molar hold-up
figure(1)
plot(t, [M_1',M_2']);
hold on
plot([5,5],[0,500],'k--',[100,100],[0,500],'k--','lineWidth',2)
ylim([0,4])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('M_1','M_2','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 50)
ylabel('Molar hold-up [kmol]', 'interpreter','latex')
set(gca,'FontSize',50);

%% Flows
figure(2)
plot(t, [F_V',F_L']);
hold on
plot([5,5],[0,500],'k--',[100,100],[0,500],'k--','lineWidth',2)
ylim([0,0.11])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('F_V','F_L','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 50)
ylabel('Molar flow [kmol/s]', 'interpreter','latex')
set(gca,'FontSize',50);

%% Phase molar hold-up
figure(3)
yyaxis left
plot(t, M_V);
ylim([0,8e-3])
ylabel('Molar hold-up [kmol]', 'interpreter','latex')
yyaxis right
plot(t, M_L);
hold on
plot([5,5],[0,500],'k--',[100,100],[0,500],'k--','lineWidth',2)
ylim([0,8])
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('M_V','M_L','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 50)
ylabel('Molar hold-up [kmol]', 'interpreter','latex')
set(gca,'FontSize',50);

%% Pressure
figure(4)
plot(t, P*10);
hold on
plot([5,5],[0,500],'k--',[100,100],[0,500],'k--','lineWidth',2)
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 50)
ylabel('Pressure, p[bar]', 'interpreter','latex')
ylim([0.6,1.2])
set(gca,'FontSize',50);

%% Temperature
figure(5)
plot(t, T*1000);
hold on
plot([5,5],[0,500],'k--',[100,100],[0,500],'k--','lineWidth',2)
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 50)
ylabel('Temperature, T[K]', 'interpreter','latex')
ylim([320,420])
set(gca,'FontSize',50);

%% Composition Vapor
figure(6)
plot(t, [y_1', y_2']);
hold on
plot([5,5],[0,1],'k--',[100,100],[0,1],'k--','lineWidth',2)
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('y_1','y_2','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 50)
ylabel('Vapor mole fraction []', 'interpreter','latex')
ylim([0,1])
set(gca,'FontSize',50);

%% Composition Vapor
figure(7)
plot(t, [x_1', x_2']);
hold on
plot([5,5],[0,1],'k--',[100,100],[0,1],'k--','lineWidth',2)
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('x_1','x_2','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 50)
ylabel('Liquid mole fraction []', 'interpreter','latex')
ylim([0,1])
set(gca,'FontSize',50);

%% Volume
figure(8)
plot(t, [V_L', V_V']);
hold on
plot([5,5],[0,1],'k--',[100,100],[0,1],'k--','lineWidth',2)
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
legend('V_V','V_L','Location','NorthEastOutside')
xlabel('t [s]','FontSize', 50)
ylabel('Volume [$m^3$]', 'interpreter','latex')
ylim([0,0.2])
set(gca,'FontSize',50);

%% Heat transfer
figure(9)
plot(t, Q);
hold on
plot([5,5],[0,-5],'k--',[100,100],[0,-5],'k--','lineWidth',2)
ax = gca;
ax.XGrid = 'off';
ax.YGrid = 'on';
xlabel('t [s]','FontSize', 50)
ylabel('Heat, Q [$MW$]', 'interpreter','latex')
ylim([-4,0])
set(gca,'FontSize',50);
end

